﻿using Model.Stub;
using System.Xml.Serialization;

namespace Model;

public class Playlist
{
    public string Name
    {
        get => name;

        set
        {
            if (value != null && value.Length < Manager.MAX_NAME_LENGTH)
            {
                name = value;
            }
        }
    }

    private string name = Manager.DEFAULT_NAME;

    public string Description
    {
        get => description;

        set
        {
            if (value != null && value.Length < Manager.MAX_DESCRIPTION_LENGTH)
            {
                description = value;
            }
        }
    }

    private string description = Manager.DEFAULT_DESC;

    private List<Title> morceaux = new List<Title>();

    public IEnumerable<Title> Morceaux {
        get 
        {
            return morceaux.ToList();
        } 
    }

    public string ImageURL
    {
        get => imageURL;

        set
        {
            if (value != null && value.Contains('.'))
            {
                imageURL = value;
            }
        }
    }

    private string imageURL = Manager.DEFAULT_URL;

    public int Index
    {
        get => index;

        set
        {
            if (value < morceaux.Count())
            {
                index = value;
            }
            else if(Loop)
            {
                index = 0;
            }
            else
            {
                return;
            }
        }
    }

    private int index = 0;

    public bool Shuffle = false;

    public bool Loop = false;

    public bool LoopTitle = false;

    Random Aleatoire = new Random();

    private List<int> played = new List<int>();

    public IEnumerable<int> Played
    {
        get
        {
            return played.ToList();
        }
    }

    public Playlist(string nom, string description, string imageURL)
    {
        Name = nom;
        Description = description;
        ImageURL = imageURL;
    }

    public Playlist()
    {
    }

    public void AddTitle(CustomTitle morceau)
    {
        morceaux.Add(morceau);
    }

    public void RemoveTitle(CustomTitle morceau)
    {
        morceaux.Remove(morceau);
    }

    public void NextTitle()
    {
        if (LoopTitle)
        {
            return;
        }

        if (!Shuffle)
        {
            Index++;
            played.Add(Index);
        }
        else
        {
            Index = Aleatoire.Next(morceaux.Count());
            played.Add(Index);
        }
    }

    public void PreviousTitle()
    {
        if (LoopTitle)
        {
            return;
        }

        if(!Shuffle)
        {
            Index--;
            played.RemoveAt(played.Count - 1);
        }
        else
        {
            if (played.Count() == 0)
            {
                return;
            }
            Index = played[played.Count - 1];
            played.RemoveAt(played.Count - 1);
        }
    }

    public Title? GetCurrentTitle()
    {
        if (Index < morceaux.Count())
        {
            return morceaux[Index];
        }
        else
        {
            return null;
        }

    }

    public override bool Equals(object? obj)
    {
        if (obj is null) return false;
        if (obj.GetType() != typeof(Playlist)) return false;
        if (obj is Playlist playlist && ImageURL == playlist.ImageURL) return true;
        else return false;
    }

    public override int GetHashCode()
    {
        return ImageURL.GetHashCode();
    }

    public override string ToString()
    {
        return $"Name : {Name}";
    }
}
