﻿namespace Model.Stub;

public class StubManager : IDataManager
{

    // Dictionaries

    IDictionary<Album, List<InfoTitle>> DictAlbumInfo
    {
        get
        {
            return dictAlbumInfo;
        }
    }

    Dictionary<Album, List<InfoTitle>> dictAlbumInfo;

    IDictionary<Playlist, List<CustomTitle>> DictPlaylistTitles
    {
        get
        {
            return dictPlaylistTitles;
        }
    }

    Dictionary<Playlist, List<CustomTitle>> dictPlaylistTitles;

    // Stubs

    public StubAlbum StubAlbum
    {
        get
        {
            return stubAlbum;
        }
    }

    private StubAlbum stubAlbum;

    public StubArtist StubArtist
    {
        get
        {
            return stubArtist;
        }
    }

    private StubArtist stubArtist;

    public StubCustomTitle StubCustomTitle
    {
        get
        {
            return stubCustomTitle;
        }
    }

    private StubCustomTitle stubCustomTitle;

    public StubInfoTitle StubInfoTitle
    {
        get
        {
            return stubInfoTitle;
        }
    }

    private StubInfoTitle stubInfoTitle;

    public StubPlaylist StubPlaylist
    {
        get
        {
            return stubPlaylist;
        }
    }

    private StubPlaylist stubPlaylist;

    public StubManager()
    {
        dictAlbumInfo = new Dictionary<Album, List<InfoTitle>>();
        dictPlaylistTitles = new Dictionary<Playlist, List<CustomTitle>>();

        stubAlbum = new StubAlbum();
        stubArtist = new StubArtist();
        stubCustomTitle = new StubCustomTitle();
        stubInfoTitle = new StubInfoTitle();
        stubPlaylist = new StubPlaylist();

        dictAlbumInfo.Add(StubAlbum.GetAlbumByUrl("lastreet.png"), StubInfoTitle.GetInfoTitlesByUrl(new List<string>() { "url1.png", "url2.png" }));
        dictAlbumInfo.Add(StubAlbum.GetAlbumByUrl("oui.png"), StubInfoTitle.GetInfoTitlesByUrl(new List<string>() { "url2.png" }));

        dictPlaylistTitles.Add(StubPlaylist.GetPlaylistByUrl("url1.png"), StubCustomTitle.GetCustomTitlesByUrl(new List<string>() { "url1.png", "url2.png" }));
        dictPlaylistTitles.Add(StubPlaylist.GetPlaylistByUrl("url2.png"), StubCustomTitle.GetCustomTitlesByUrl(new List<string>() { "url2.png", "url3.png" }));
    }

    public List<Album> GetAlbums()
    {
        return StubAlbum.GetAlbums();   
    }

    public List<Artist> GetArtists()
    {
        return StubArtist.GetArtists();
    }

    public List<Playlist> GetPlaylists()
    {
        return StubPlaylist.GetPlaylists();
    }

    public List<CustomTitle> GetCustomTitles()
    {
        return StubCustomTitle.GetCustomTitles();
    }

    public List<InfoTitle> GetInfoTitles()
    {
        return StubInfoTitle.GetInfoTitles();
    }

    public void AddAlbum(Album album)
    {
        StubAlbum.AddAlbum(album);
    }

    public void AddCustomTitle(CustomTitle title)
    {
        StubCustomTitle.AddCustomTitle(title);
    }

    public void AddInfoTitle(InfoTitle title)
    {
        StubInfoTitle.AddInfoTitle(title);
    }

    public void AddFeat(InfoTitle infoTitle, Artist artist)
    {
        StubInfoTitle.AddFeat(infoTitle, artist);
    }

    public void AddPlaylist(Playlist playlist)
    {
        StubPlaylist.AddPlaylist(playlist);
    }

    public void AddArtist(Artist artist)
    {
        StubArtist.AddArtist(artist);
    }



    public void RemoveAlbum(Album album)
    {
        StubAlbum.RemoveAlbum(album);
    }

    public void RemoveCustomTitle(CustomTitle title)
    {
        StubCustomTitle.RemoveCustomTitle(title);
    }

    public void RemoveInfoTitle(InfoTitle title)
    {
        StubInfoTitle.RemoveInfoTitle(title);
    }

    public void RemovePlaylist(Playlist playlist)
    {
        StubPlaylist.RemovePlaylist(playlist);
    }

    public void RemoveArtist(Artist artist)
    {
        StubArtist.RemoveArtist(artist);
    }





    public void LoadSerialization()
    {
        return;
    }

    public void SaveSerialization()
    {
        return;
    }

    public CustomTitle? GetCustomTitleByUrl(string custom)
    {
        foreach (CustomTitle customTitle in StubCustomTitle.GetCustomTitles())
        {
            if (customTitle.Name == custom)
            {
                return customTitle;
            }
        }
        return null;
    }
}
